@@ -4,7 +4,7 @@
  * GChemPaint library
  * atom.h 
  *
- * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,6 +28,7 @@
 #include <map>
 #include <glib.h>
 #include <gccv/item-client.h>
+#include <gccv/structs.h>
 #include <gcu/atom.h>
 #include <gcu/dialog-owner.h>
 #include <gcu/element.h>
@@ -182,15 +183,10 @@ arranged.
 On input \a Pos can be one of POSITION_E, POSITION_N,... or 0xff, in which case,
 it will be given a default value. \a x and \a y are set to the position where the charge
 sign should be displayed usding the alignment code returned by this method.
-@return a number to set how the charge symbol should be aligned relative to its
-position. Possible values are:
-- −2: center top.
-- −1: right.
--  0: center.
--  1: left.
--  2: center bottom.
+@return the anchor for the charge symbol. On error, gccv::AnchorCenter is used as
+the returned value.
 */
-	virtual int GetChargePosition (unsigned char& Pos, double Angle, double& x, double& y);
+	virtual gccv::Anchor GetChargePosition (unsigned char& Pos, double Angle, double& x, double& y);
 /*!
 @param x the x position.
 @param y the y position.
@@ -206,6 +202,16 @@ that they give the absolute position.
 @param x the x position.
 @param y the y position.
 
+Updates \a x and \a y so that they become the position corresponding to the angle
+when the position is available, the origin being the position of the atom.
+@return true on success, false otherwise.
+*/
+	virtual bool GetRelativePosition (double angle, double& x, double& y);
+/*!
+@param angle the angle at which a charge sign or an electron should be displayed.
+@param x the x position.
+@param y the y position.
+
 Updates \a x and \a y so that they become the absolute position corresponding to the angle
 when the position is available.
 @return true on success, false otherwise.
@@ -444,6 +450,8 @@ Sets the position of attached hydrogen atoms symbol.
 @return the position of attached hydrogen atoms symbol as a reference.
 */
 GCU_PROP (HPos, HPosStyle) //0=force left, 1=force right, 2=force top, 3=force bottom, 4=auto.
+GCU_PROP (gccv::Item *, ChargeItem) //0=force left, 1=force right, 2=force top, 3=force bottom, 4=auto.
+GCU_PROP (bool, ShowCharge);
 };
 
 }	//	namespace gcp
