@@ -152,7 +152,7 @@ void Document::Clear ()
 
 GtkWidget* Document::GetWidget ()
 {
-	return (m_pView) ? m_pView->GetWidget () : NULL;
+	return (m_pView)? m_pView->GetWidget (): NULL;
 }
 
 
@@ -160,8 +160,9 @@ const gchar* Document::GetTitle () const
 {
 	if (m_title)
 		return m_title;
-	else
-		return GetLabel ();
+	else if (m_label)
+		return m_label;
+	return (m_Window)? m_Window->GetDefaultTitle (): NULL;
 }
 
 const gchar* Document::GetLabel () const
@@ -358,7 +359,7 @@ void Document::ExportOB () const
 				Mol.SetTitle ((char*) GetTitle ());
 				Mol.SetDimension (2);
 				for (i = AtomTable.begin (); i != AtomTable.end (); i++) {
-					pgAtom = (Atom*) Ob->GetDescendant ((*i).first.data ());
+					pgAtom = (Atom*) Ob->GetDescendant ((*i).first.c_str ());
 					obAtom.SetIdx ((*i).second);
 					obAtom.SetAtomicNum (pgAtom->GetZ ());
 					pgAtom->GetCoords (&x, &y, &z);
@@ -768,12 +769,13 @@ bool Document::Load (xmlNodePtr root)
 		pObject = CreateObject ((const char*) child->name, this);
 		if (pObject) {
 			if (!pObject->Load (child))
-				delete pObject;
+				Remove (pObject);
 			else
 				m_pView->AddObject (pObject);
 		}
 		node = node->next;
 	}
+	Loaded ();
 	m_pView->Update (this);
 	Update ();
 	m_Empty = !HasChildren ();
@@ -1224,6 +1226,7 @@ void Document::LoadObjects (xmlNodePtr node)
 		else delete pBond;
 		child = GetNextNodeByName (child->next, "bond");
 	}
+	Loaded ();
 }
 
 Operation* Document::GetNewOperation (OperationType type)
@@ -1265,6 +1268,7 @@ void Document::AddData (xmlNodePtr node)
 	}
 	m_bIsLoading = false;
 	EmptyTranslationTable ();
+	Loaded ();
 	FinishOperation ();
 }
 
@@ -1409,7 +1413,7 @@ double Document::GetMedianBondLength ()
 		}
 	}
 	if (nb > 0)
-		go_range_median_inter_nonconst (lengths.data (), nb, &result);
+		go_range_median_inter_nonconst (&lengths[0], nb, &result);
 	return result;
 }
 
