@@ -4,7 +4,7 @@
  * GChemPaint library
  * fragment.cc 
  *
- * Copyright (C) 2002-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -178,16 +178,28 @@ bool Fragment::OnChanged (bool save)
 	Document* pDoc = (Document*) GetDocument ();
 	if (!pDoc)
 		return false;
+	int length = m_buf.length ();
 	m_buf = m_TextItem->GetText ();
+	if (length)
+		length -= m_buf.length ();
 	View* pView = pDoc->GetView ();
 	unsigned CurPos = m_TextItem->GetCursorPosition ();
 	AnalContent (m_StartSel, CurPos);
 	m_bLoading = true;
 	/*main atom management*/
-	if (m_EndSel > CurPos && m_BeginAtom >= m_EndSel) {
-		// if text was deleted before the atom, adjust its bounds
-		m_BeginAtom -= m_EndSel - CurPos;
-		m_EndAtom -= m_EndSel - CurPos;
+	if (m_EndAtom > m_BeginAtom) {
+		if (m_EndSel > CurPos && m_BeginAtom >= m_EndSel) {
+			// if text was deleted before the atom, adjust its bounds
+			m_BeginAtom -= m_EndSel - CurPos;
+			m_EndAtom -= m_EndSel - CurPos;
+		} else if (length > 0 && CurPos < m_BeginAtom + length) {
+			if (m_BeginAtom >= static_cast <unsigned> (length)) {
+				m_BeginAtom -= length;
+				m_EndAtom -= length;
+			} else {
+				m_BeginAtom = m_EndAtom = 0;
+			}
+		}
 	}
 	if (m_buf.length () < m_EndAtom) { // needed if the symbol of part of it has been destroyed
 		m_Atom->SetZ (0);
@@ -269,8 +281,10 @@ bool Fragment::OnChanged (bool save)
 		}
 	} else if (m_EndSel <= m_BeginAtom) {
 		int delta = CurPos - m_EndSel;
-		m_BeginAtom += delta;
-		m_EndAtom += delta;
+		if (delta > 0) { // negative values have already been dealt with
+			m_BeginAtom += delta;
+			m_EndAtom += delta;
+		}
 	} else if ((m_EndAtom <= m_EndSel && m_EndAtom >= m_StartSel) ||
 		(m_BeginAtom <= m_EndSel && m_BeginAtom >= m_StartSel) ||
 		(m_BeginAtom + Residue::MaxSymbolLength >= CurPos)) {
@@ -868,18 +882,23 @@ void Fragment::AnalContent (unsigned start, unsigned &end)
 					new_tag->SetStartIndex (start);
 					new_tag->SetEndIndex (next);
 					Stoich = true;
-				}
-			} else if (start == tag->GetEndIndex () - 1) {
+				} else
+					tag->SetEndIndex (next);
+			} else if (start == tag->GetEndIndex ()) {
 				string repl (1, c);
 				char *buf = g_utf8_find_prev_char (text, text + start);
 				repl.append (buf, text + start - buf);
+				new_tag = new ChargeTextTag (size);
+				new_tag->SetStartIndex (tag->GetStartIndex ());
+				new_tag->SetEndIndex (next);
 				if (m_TextItem) {
 					m_TextItem->ReplaceText (repl, buf - text, text + start + 1 - buf);
 					m_buf = m_TextItem->GetText ();
 				} else
 					m_buf.replace (buf - text, repl.length (), repl.c_str ());
 				text = m_buf.c_str ();
-			}
+			} else
+				tag->SetEndIndex (tag->GetEndIndex () + 1);
 			if (new_tag) {
 				if (m_TextItem)
 					m_TextItem->InsertTextTag (new_tag);
@@ -921,43 +940,47 @@ void Fragment::AnalContent (unsigned start, unsigned &end)
 					}
 				} else {
 					// old charge is tag content minus the last character (+ or - just typed)
-					string old_charge (m_buf, tag->GetStartIndex (), tag->GetEndIndex () - tag->GetStartIndex () - 1);
+					string old_charge (m_buf, tag->GetStartIndex (), tag->GetEndIndex () - tag->GetStartIndex ());
 					char *nextch = NULL;
 					int charge = strtol (old_charge.c_str (), &nextch, 10);
 					if (charge == 0)
 						charge = 1;
 					if (nextch && !strncmp (nextch, "−", lenminus))
 						charge = -charge;
-					if (*(text + tag->GetEndIndex () - 1) == '+')
+					if (*(text + tag->GetEndIndex ()) == '+')
 						charge++;
 					else
 						charge--;
 					if (charge == 0) {
 						old_charge.clear ();
-						end -= tag->GetEndIndex () - tag->GetStartIndex ();
+						end -= tag->GetEndIndex () - tag->GetStartIndex () + 1;
 						next = tag->GetStartIndex ();
 						if (m_BeginAtom > start) {
 							m_BeginAtom -= tag->GetEndIndex () - tag->GetStartIndex ();
 							m_EndAtom -= tag->GetEndIndex () - tag->GetStartIndex ();
 						}
-						m_TextItem->ReplaceText (old_charge, tag->GetStartIndex (), tag->GetEndIndex () - tag->GetStartIndex ());
+						m_TextItem->ReplaceText (old_charge, tag->GetStartIndex (), tag->GetEndIndex () - tag->GetStartIndex () + 1);
 						m_buf = m_TextItem->GetText ();
 						text = m_buf.c_str ();
 						tag = NULL;
 						Charge = false;
 					} else {
-						nextch = g_strdup_printf ("%d", abs (charge));
+						nextch = (abs (charge) > 1)? g_strdup_printf ("%d", abs (charge)): g_strdup ("");
 						old_charge = string (nextch) + ((charge > 0)? "+": "−");
-						end -= tag->GetEndIndex () - tag->GetStartIndex () - old_charge.length ();
+						end -= tag->GetEndIndex () - tag->GetStartIndex () - old_charge.length () + 1;
+						start = tag->GetStartIndex ();
 						next = start + old_charge.length ();
 						if (m_BeginAtom > start) {
 							m_BeginAtom -= tag->GetEndIndex () - tag->GetStartIndex () - old_charge.length ();
 							m_EndAtom -= tag->GetEndIndex () - tag->GetStartIndex () - old_charge.length ();
 						}
-						m_TextItem->ReplaceText (old_charge, tag->GetStartIndex (), tag->GetEndIndex () - tag->GetStartIndex ());
+						m_TextItem->ReplaceText (old_charge, tag->GetStartIndex (), tag->GetEndIndex () - tag->GetStartIndex () + 1);
 						m_buf = m_TextItem->GetText ();
 						text = m_buf.c_str ();
 						g_free (nextch);
+						new_tag = new ChargeTextTag (size);
+						new_tag->SetStartIndex (start);
+						new_tag->SetEndIndex (next);
 					}
 				}
 				if (new_tag) {
@@ -1518,4 +1541,9 @@ gccv::Item *Fragment::GetChargeItem ()
 	return (m_Atom)? m_Atom->GetChargeItem (): NULL;
 }
 
+std::string Fragment::Name ()
+{
+	return _("Fragment");
+}
+
 }	//	namespace gcp
