@@ -4,7 +4,7 @@
  * GChemPaint library
  * reaction-operator.h 
  *
- * Copyright (C) 2004-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -54,10 +54,6 @@ Used to add a representation of the operator in the view.
 */
 	void AddItem ();
 /*!
-Used to update the representation of the operator in the view.
-*/
-	void UpdateItem ();
-/*!
 @param x the x component of the transation vector.
 @param y the y component of the transation vector.
 @param z the z component of the transation vector (unused).
@@ -93,7 +89,9 @@ Used to retrieve the y coordinate for alignment.
 @return y coordinate used for reaction operators alignment.
 */
 	virtual double GetYAlign ();
-	
+
+	std::string Name ();
+
 private:
 	double m_x, m_y;
 	PangoLayout *m_Layout;
