@@ -150,7 +150,7 @@ ContentType CIFLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 				prop = KnownProps.find (key);
 				if (prop != KnownProps.end ())
 					doc->SetProperty ((*prop).second, value.c_str ());
-				// unkown data are discarded
+				// unknown data are discarded
 				waiting_value = false;
 			} else {
 				if (value.length () > 0 && strlen (cur) > 0)
@@ -191,6 +191,9 @@ ContentType CIFLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 					if (in_loop) {
 						prop = KnownProps.find (key);
 						loop_contents.push_back ((prop == KnownProps.end ())? static_cast <unsigned> (GCU_PROP_MAX): (*prop).second);
+						// initialize loop_prop there because loop_ might be erroneously followed by
+						// just a property and its value.
+						loop_prop = loop_contents.begin ();
 						if (loop_type == LOOP_UNKNOWN) {
 							if (!key.compare (0, 11, "_atom_type_", 11))
 								loop_type = LOOP_ATOM_TYPE;
@@ -222,6 +225,7 @@ ContentType CIFLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 					continue; // FIXME: this should not happen
 				if (*cur == 0) {
 					in_loop = false;
+					waiting_value = false;
 				} else {
 					while (*cur) {
 						while (*cur == ' ')
@@ -247,6 +251,7 @@ ContentType CIFLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 						*next = endstr;
 						cur = next;
 						loop_prop++;
+						waiting_value = true; // fixes a loop with just a key and value on the same line
 						loop_values.push_back (value);
 						if (loop_prop == loop_contents.end ()) {
 							// store the values
@@ -270,9 +275,13 @@ ContentType CIFLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 										break;
 									}
 									case CIF_ATOM_SITE_SYMBOL: {
-										CIFAtomType t = AtomTypes[val];
-										atom->SetProperty (GCU_PROP_ATOM_Z, t.elt.c_str ());
-										atom->SetProperty (GCU_PROP_ATOM_CHARGE, t.charge.c_str ());
+										if (AtomTypes.empty ())
+											atom->SetProperty (GCU_PROP_ATOM_SYMBOL, val.c_str ());
+										else {
+											CIFAtomType t = AtomTypes[val];
+											atom->SetProperty (GCU_PROP_ATOM_Z, t.elt.c_str ());
+											atom->SetProperty (GCU_PROP_ATOM_CHARGE, t.charge.c_str ());
+										}
 										break;
 									}
 									default:
@@ -375,24 +384,37 @@ ContentType CIFLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 				prop = KnownProps.find (key);
 				if (prop != KnownProps.end ())
 					doc->SetProperty ((*prop).second, value.c_str ());
-				// unkown properties are discarded
+				// unknown properties are discarded
 			}
 			waiting_value = false;
 			continue;
 		}
 	}
-	// check ymmetry
+	// check symmetry
 	if (!group->IsValid ()) {
-		GtkWidget *w = gtk_message_dialog_new (doc->GetGtkWindow (), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Invalid symmetry group."));
-		g_signal_connect (G_OBJECT (w), "response", G_CALLBACK (gtk_widget_destroy), NULL);
+		if (group->GetTransformsNumber ()) {
+			GtkWidget *w = gtk_message_dialog_new (doc->GetGtkWindow (), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Invalid symmetry group."));
+			g_signal_connect (G_OBJECT (w), "response", G_CALLBACK (gtk_widget_destroy), NULL);
 		gtk_widget_show_all (w);
+		}
 	} else {
 		SpaceGroup const *sp = SpaceGroup::Find (group);
 		if (sp)
 			doc->SetProperty (GCU_PROP_SPACE_GROUP, sp->GetHallName ().c_str ());
+		else {
+			group->RegisterSpaceGroup (0);
+			std::string name = group->GetHallName ();
+			if (name.length () == 0)
+				name = group->GetHMName ();
+			if (name.length () > 0)
+				doc->SetProperty (GCU_PROP_SPACE_GROUP, name.c_str ());
+			// FIXME: implement use of Id if available
+			group = NULL; // do not delete
+		}
 	}
 read_exit:
-	delete group;
+	if (group)
+		delete group;
 	g_object_unref (input);
 	return type;
 }
