@@ -38,9 +38,11 @@ public:
 	virtual ~gc3dWindow ();
 
 	void OnFileOpen ();
-	void OnFilePrint ();
+	void OnPageSetup ();
 	void OnFileClose ();
 	void SetTitle (char const *title);
+	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context);
+	GtkWindow *GetWindow () {return m_Window;}
 
 private:
 	GtkWindow* m_Window;
