@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -30,6 +30,7 @@
 #include <gcu/chemistry.h>
 #include <gcu/element.h>
 #include <gcu/filechooser.h>
+#include <gcu/ui-builder.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtkaboutdialog.h>
@@ -45,105 +46,118 @@
 
 using namespace std;
 
-static void on_quit (GtkWidget *widget, GChemTableApp *App)
+static void on_quit (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
 	delete App;
 	gtk_main_quit();
 }
 
-static void on_new_chart (GtkWidget *widget, GChemTableApp *App)
+static void on_new_chart (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
-	App->OnNewChart ();
+	App->OnNewChart (NULL);
 }
 
-static void on_no_colors (GtkWidget *widget, GChemTableApp *App)
+static void on_electroneg_chart (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
+{
+	App->OnNewChart ("en/Pauling");
+}
+
+static void on_radius_chart (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
+{
+	App->OnNewChart ("covalent");
+}
+
+static void on_affinity_chart (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
+{
+	App->OnNewChart ("ae");
+}
+
+static void on_no_colors (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("none");
 }
 
-void on_default_colors (GtkWidget *widget, GChemTableApp *App)
+void on_default_colors (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("default");
 }
 
-static void on_state_colors (GtkWidget *widget, GChemTableApp *App)
+static void on_state_colors (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("state");
 }
 
-static void on_family_colors (GtkWidget *widget, GChemTableApp *App)
+static void on_family_colors (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("family");
 }
 
-void on_acidity_colors (GtkWidget *widget, GChemTableApp *App)
+void on_acidity_colors (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("acidity");
 }
 
-void on_electroneg_colors (GtkWidget *widget, GChemTableApp *App)
+void on_electroneg_colors (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("electroneg");
 }
 
-static void on_radius_colors (GtkWidget *widget, GChemTableApp *App)
+static void on_radius_colors (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("radius");
 }
 
-static void on_block_colors (GtkWidget *widget, GChemTableApp *App)
+static void on_block_colors (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *App)
 {
 	App->SetColorScheme ("block");
 }
 
-static void on_about_activate_url (GtkAboutDialog *about, const gchar *url, gpointer data)
+static void on_about_activate_url (G_GNUC_UNUSED GtkAboutDialog *about, const gchar *url, gpointer data)
 {
 	reinterpret_cast <Application *> (data)->OnWeb (url);
 }
 
-static void on_help (GtkWidget *widget, GChemTableApp *app)
+static void on_help (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
 {
 	app->OnHelp ();
 }
 
-static void on_web (GtkWidget *widget, GChemTableApp *app)
+static void on_web (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
 {
 	app->OnWeb ();
 }
 
-static void on_mail (GtkWidget *widget, GChemTableApp *app)
+static void on_mail (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
 {
 	app->OnMail ();
 }
 
-static void on_live_assistance (GtkWidget *widget, GChemTableApp *app)
+static void on_live_assistance (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
 {
 	app->OnLiveAssistance ();
 }
 
-static void on_bug (GtkWidget *widget, GChemTableApp *app)
+static void on_bug (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
 {
 	app->OnBug ();
 }
 
-static void on_about (GtkWidget *widget, GChemTableApp *app)
+static void on_about (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
 {
 	app->OnAbout ();
 }
 
-static void on_changed (GtkPeriodic* periodic, guint Z, GChemTableApp *app)
+static void on_changed (G_GNUC_UNUSED GcuPeriodic* periodic, guint Z, GChemTableApp *app)
 {
 	app->OnElement (Z);
 }
 
 static GtkActionEntry entries[] = {
-  { "FileMenu", NULL, N_("_File") },
-	  { "NewChart", NULL, N_("New _Chart"), "<control>N",
-		  N_("Create a new chart"), G_CALLBACK (on_new_chart) },
+  { "FileMenu", NULL, N_("_File"), NULL, NULL, NULL },
 	  { "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
 		  N_("Quit GChemTable"), G_CALLBACK (on_quit) },
-  { "ViewMenu", NULL, N_("_View") },
-    {"ColorMenu", NULL, N_("Color scheme") },
+  { "ViewMenu", NULL, N_("_View"), NULL, NULL, NULL },
+    {"ColorMenu", NULL, N_("Color scheme"), NULL, NULL, NULL },
 	  {"NoColors", NULL, N_("No colors"), NULL,
 		  N_("Use default Gtk theme colors"), G_CALLBACK (on_no_colors) },
 	  {"DefaultColors", NULL, N_("Default"), NULL,
@@ -166,7 +180,16 @@ static GtkActionEntry entries[] = {
 	  {"BlockColors", NULL, N_("Block"), NULL,
 		  N_("Use colors to display the blocks elements belong to"),
                   G_CALLBACK (on_block_colors) },
-  { "HelpMenu", NULL, N_("_Help") },
+    {"ChartMenu", NULL, N_("Element charts "), NULL, NULL, NULL },
+      { "ElectronegChart", NULL, N_("Electronegativty "), NULL,
+		  N_("Create an electronegativity chart"), G_CALLBACK (on_electroneg_chart) },
+      { "RadiusChart", NULL, N_("Atomic radius"), NULL,
+		  N_("Create an atomic radius chart"), G_CALLBACK (on_radius_chart) },
+      { "AffinityChart", NULL, N_("Electon affinity"), NULL,
+		  N_("Create an electronic affinity chart"), G_CALLBACK (on_affinity_chart) },
+	  { "CustomChart", NULL, N_("Custom "), NULL,
+		  N_("Create a custom chart"), G_CALLBACK (on_new_chart) },
+  { "HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL },
 	  { "Help", GTK_STOCK_HELP, N_("_Contents"), "F1",
 		  N_("View help for the Periodic Table"), G_CALLBACK (on_help) },
 	  { "Web", NULL, N_("Gnome Chemistry Utils on the _web"), NULL,
@@ -185,9 +208,6 @@ static const char *ui_description =
 "<ui>"
 "  <menubar name='MainMenu'>"
 "    <menu action='FileMenu'>"
-"	   <separator name='file-sep1'/>"
-"      <menuitem action='NewChart'/>"
-"	   <separator name='file-sep2'/>"
 "      <menuitem action='Quit'/>"
 "    </menu>"
 "    <menu action='ViewMenu'>"
@@ -201,6 +221,12 @@ static const char *ui_description =
 "        <menuitem action='RadiusColors'/>"
 "        <menuitem action='BlockColors'/>"
 "      </menu>"
+"      <menu action='ChartMenu'>"
+"        <menuitem action='ElectronegChart'/>"
+"        <menuitem action='RadiusChart'/>"
+"        <menuitem action='AffinityChart'/>"
+"        <menuitem action='CustomChart'/>"
+"      </menu>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
 "      <menuitem action='Help'/>"
@@ -288,52 +314,56 @@ GChemTableApp::GChemTableApp (): Application ("gchemtable")
 	}
 	GtkWidget *bar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
 	gtk_box_pack_start (GTK_BOX (vbox), bar, FALSE, FALSE, 0);
-	periodic = GTK_PERIODIC (gtk_periodic_new());
+	periodic = GCU_PERIODIC (gcu_periodic_new());
 	g_object_set(G_OBJECT(periodic),
-			"color-style", GTK_PERIODIC_COLOR_DEFAULT,
+			"color-style", GCU_PERIODIC_COLOR_DEFAULT,
 			"can_unselect", true,
 			NULL);
 	g_signal_connect(G_OBJECT(periodic), "element_changed", (GCallback)on_changed, this);
-	gtk_box_pack_end_defaults(GTK_BOX(vbox), GTK_WIDGET(GTK_PERIODIC(periodic)));
+	gtk_box_pack_end (GTK_BOX(vbox), GTK_WIDGET(GCU_PERIODIC(periodic)), true, true, 0);
 	gtk_container_add(GTK_CONTAINER(window), GTK_WIDGET(vbox));
 	gtk_widget_show_all(window);
 	for (int i = 0; i < 118; i++)
 		Pages[i] = NULL;
 	
 	gcu::Element::LoadAllData ();
-	colorschemes["none"] = GTK_PERIODIC_COLOR_NONE;
-	colorschemes["default"] = GTK_PERIODIC_COLOR_DEFAULT;
+	colorschemes["none"] = GCU_PERIODIC_COLOR_NONE;
+	colorschemes["default"] = GCU_PERIODIC_COLOR_DEFAULT;
 
-	GladeXML *xml = glade_xml_new (GLADEDIR"/state-thermometer.glade", "state-thermometer", NULL);
-	GtkWidget *thermometer = glade_xml_get_widget (xml, "state-thermometer");
-	colorschemes["state"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_state_color, thermometer, this);
+	UIBuilder *builder = new UIBuilder (UIDIR"/state-thermometer.ui", GETTEXT_PACKAGE);
+	GtkWidget *thermometer = builder->GetRefdWidget ("state-thermometer");
+	colorschemes["state"] = gcu_periodic_add_color_scheme (periodic, (GcuPeriodicColorFunc) get_state_color, thermometer, this);
 	gtk_widget_show_all (thermometer);
-	thermometer = glade_xml_get_widget (xml, "temperature");
+	thermometer = builder->GetWidget ("temperature");
 	g_signal_connect (G_OBJECT (thermometer), "value-changed", G_CALLBACK (on_changed_temp), this);
 	temperature = gtk_range_get_value (GTK_RANGE (thermometer));
+	delete builder;
 
-	GladeXML *familyxml = glade_xml_new (GLADEDIR"/family.glade", "family-legend", NULL);
-	GtkWidget *familywidget = glade_xml_get_widget (familyxml, "family-legend");
-	colorschemes["family"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_family_color, familywidget, this);	
+	builder = new UIBuilder (UIDIR"/family.ui", GETTEXT_PACKAGE);
+	GtkWidget *familywidget = builder->GetRefdWidget ("family-legend");
+	colorschemes["family"] = gcu_periodic_add_color_scheme (periodic, (GcuPeriodicColorFunc) get_family_color, familywidget, this);	
 	gtk_widget_show_all (familywidget);
-	familywidget = glade_xml_get_widget (familyxml, "family-box");
+	familywidget = builder->GetWidget ("family-box");
 	gtk_combo_box_set_active (GTK_COMBO_BOX(familywidget), 0);
 	family = -1;
 	g_signal_connect (G_OBJECT (familywidget), "changed", G_CALLBACK (on_changed_family), this);
+	delete builder;
 
-	GladeXML *acidityxml = glade_xml_new (GLADEDIR"/acidity.glade", "acidity-legend", NULL);
-	GtkWidget *aciditylegend = glade_xml_get_widget (acidityxml, "acidity-legend");
-	colorschemes["acidity"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_acidity_color, aciditylegend, this);
+	builder = new UIBuilder (UIDIR"/acidity.ui", GETTEXT_PACKAGE);
+	GtkWidget *aciditylegend = builder->GetRefdWidget ("acidity-legend");
+	colorschemes["acidity"] = gcu_periodic_add_color_scheme (periodic, (GcuPeriodicColorFunc) get_acidity_color, aciditylegend, this);
 	gtk_widget_show_all (aciditylegend);
+	delete builder;
 
-	colorschemes["electroneg"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_electroneg_color, NULL, this);
+	colorschemes["electroneg"] = gcu_periodic_add_color_scheme (periodic, (GcuPeriodicColorFunc) get_electroneg_color, NULL, this);
 
-	colorschemes["radius"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_radius_color, NULL, this);
+	colorschemes["radius"] = gcu_periodic_add_color_scheme (periodic, (GcuPeriodicColorFunc) get_radius_color, NULL, this);
 
-	GladeXML *blockxml = glade_xml_new (GLADEDIR"/block.glade", "block-legend", NULL);
-	GtkWidget *blocklegend = glade_xml_get_widget (blockxml, "block-legend");
-	colorschemes["block"] = gtk_periodic_add_color_scheme (periodic, (GtkPeriodicColorFunc) get_block_color, blocklegend, this);
+	builder = new UIBuilder (UIDIR"/block.ui", GETTEXT_PACKAGE);
+	GtkWidget *blocklegend = builder->GetRefdWidget ("block-legend");
+	colorschemes["block"] = gcu_periodic_add_color_scheme (periodic, (GcuPeriodicColorFunc) get_block_color, blocklegend, this);
 	gtk_widget_show_all (blocklegend);
+	delete builder;
 	gct_data_init ();
 
 }
@@ -360,7 +390,7 @@ void GChemTableApp::OnAbout ()
 		"GNU General Public License for more details.\n\n"
 		"You should have received a copy of the GNU General Public License\n"
 		"along with this program; if not, write to the Free Software\n"
-		"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307\n"
+		"Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301\n"
 		"USA";
 	
 	gtk_about_dialog_set_url_hook (on_about_activate_url, this, NULL);
@@ -394,7 +424,7 @@ void GChemTableApp::OnElement (int Z)
 void GChemTableApp::ClearPage (int Z)
 {
 	Pages[Z - 1] = NULL;
-	gtk_periodic_set_element (periodic, 0);
+	gcu_periodic_set_element (periodic, 0);
 	if (Z == m_CurZ) {
 		SetCurZ (0);
 	}
@@ -403,7 +433,7 @@ void GChemTableApp::ClearPage (int Z)
 void GChemTableApp::SetCurZ (int Z)
 {
 	if (Z != m_CurZ) {
-		gtk_periodic_set_element (periodic, Z);
+		gcu_periodic_set_element (periodic, Z);
 		m_CurZ = Z;
 	}
 }
@@ -416,13 +446,13 @@ void GChemTableApp::SetColorScheme (char const *name)
 void GChemTableApp::SetTemperature (double T)
 {
 	temperature = T;
-	gtk_periodic_set_colors (periodic);
+	gcu_periodic_set_colors (periodic);
 }
 
 void GChemTableApp::SetFamily (int family_N)
 {
 	family = (family_N)? 1 << (family_N - 1): -1;
-	gtk_periodic_set_colors (periodic);
+	gcu_periodic_set_colors (periodic);
 }
 
 void GChemTableApp::GetStateColor (int Z, GdkColor *color)
@@ -646,9 +676,9 @@ void GChemTableApp::GetBlockColor (int Z, GdkColor *color)
 	}
 }
 
-void GChemTableApp::OnNewChart ()
+void GChemTableApp::OnNewChart (char const *name)
 {
-	new GChemTableCurve (this, NULL);
+	new GChemTableCurve (this, name);
 }
 
 void GChemTableApp::OnSaveAsImage (GChemTableCurve *curve)
@@ -656,7 +686,6 @@ void GChemTableApp::OnSaveAsImage (GChemTableCurve *curve)
 	if (!curve)
 		return;
 	list<string> l;
-	unsigned n = 0;
 	char const *mime;
 	map<string, GdkPixbufFormat*>::iterator i, end = m_SupportedPixbufFormats.end ();
 	for (i = m_SupportedPixbufFormats.begin (); i != end; i++)
@@ -672,7 +701,7 @@ void GChemTableApp::OnSaveAsImage (GChemTableCurve *curve)
 	FileChooser (this, true, l, reinterpret_cast <Document *> (curve), _("Save as image"), GetImageSizeWidget ());
 }
 
-bool GChemTableApp::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *Doc)
+bool GChemTableApp::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, G_GNUC_UNUSED GtkWindow *window, Document *Doc)
 {
 	GChemTableCurve *curve = reinterpret_cast <GChemTableCurve *> (Doc);
 	if(bSave) {
